@@ -5,7 +5,7 @@ from os import environ
 from typing import AsyncGenerator
 
 import websockets.client as websockets
-from aiohttp import ClientSession
+from aiohttp import ClientSession, TCPConnector
 from websockets.client import WebSocketClientProtocol
 
 from sydney.constants import (
@@ -20,14 +20,23 @@ from sydney.enums import (
     ComposeTone,
     ConversationStyle,
     MessageType,
+    ResultValue,
+)
+from sydney.exceptions import (
+    CaptchaChallengeException,
+    NoConnectionException,
+    NoResponseException,
+    ThrottledRequestException,
 )
-from sydney.exceptions import NoConnectionException, NoResponseException
 from sydney.utils import as_json
 
 
 class SydneyClient:
     def __init__(
-        self, style: str = "balanced", bing_u_cookie: str | None = None
+        self,
+        style: str = "balanced",
+        bing_u_cookie: str | None = None,
+        use_proxy: bool = False,
     ) -> None:
         """
         Client for Bing Chat.
@@ -40,10 +49,15 @@ class SydneyClient:
         bing_u_cookie: str | None
             The _U cookie from Bing required to connect and use Bing Chat. If not provided,
             the `BING_U_COOKIE` environment variable is loaded instead. Default is None.
+        use_proxy: str | None
+            Flag to determine if an HTTP proxy will be used to start a conversation with Bing Chat. If set to True,
+            the `HTTP_PROXY` and `HTTPS_PROXY` environment variables must be set to the address of the proxy to be used.
+            If not provided, no proxy will be used. Default is False.
         """
         self.bing_u_cookie = (
             bing_u_cookie if bing_u_cookie else environ["BING_U_COOKIE"]
         )
+        self.use_proxy = use_proxy
         self.conversation_style: ConversationStyle = getattr(
             ConversationStyle, style.upper()
         )
@@ -61,19 +75,23 @@ class SydneyClient:
         await self.close_conversation()
 
     def _build_ask_arguments(self, prompt: str) -> dict:
+        style_options = self.conversation_style.value.split(",")
+        options_sets = [
+            "nlu_direct_response_filter",
+            "deepleo",
+            "disable_emoji_spoken_text",
+            "responsible_ai_policy_235",
+            "enablemm",
+            "dv3sugg",
+        ]
+        for style in style_options:
+            options_sets.append(style.strip())
+
         return {
             "arguments": [
                 {
                     "source": "cib",
-                    "optionsSets": [
-                        "nlu_direct_response_filter",
-                        "deepleo",
-                        "disable_emoji_spoken_text",
-                        "responsible_ai_policy_235",
-                        "enablemm",
-                        "dv3sugg",
-                        self.conversation_style.value,
-                    ],
+                    "optionsSets": options_sets,
                     "isStartOfSession": self.invocation_id == 0,
                     "message": {
                         "author": "user",
@@ -136,9 +154,10 @@ class SydneyClient:
         self,
         prompt: str,
         citations: bool = False,
+        suggestions: bool = False,
         raw: bool = False,
         stream: bool = False,
-    ) -> AsyncGenerator[str | dict, None]:
+    ) -> AsyncGenerator[tuple[str | dict, list | None], None]:
         if (
             self.conversation_id is None
             or self.client_id is None
@@ -166,24 +185,48 @@ class SydneyClient:
                     continue
                 response = json.loads(obj)
                 # Handle type 1 messages when streaming is enabled.
-                if (
-                    stream
-                    and response.get("type") == 1
-                    and response["arguments"][0].get("messages")
-                ):
+                if stream and response.get("type") == 1:
+                    messages = response["arguments"][0].get("messages")
+                    # Skip on empty response.
+                    if not messages:
+                        continue
+
                     if raw:
-                        yield response
+                        yield response, None
                     elif citations:
-                        yield response["arguments"][0]["messages"][0]["adaptiveCards"][0]["body"][0]["text"]
+                        yield messages[0]["adaptiveCards"][0]["body"][0]["text"], None
                     else:
-                        yield response["arguments"][0]["messages"][0]["text"]
+                        yield messages[0]["text"], None
                 # Handle type 2 messages.
                 elif response.get("type") == 2:
+                    messages = response["item"].get("messages")
+                    if not messages:
+                        result_value = response["item"]["result"]["value"]
+                        # Throttled - raise error.
+                        if result_value == ResultValue.THROTTLED.value:
+                            raise ThrottledRequestException("Request is throttled")
+                        # Captcha chalennge - user needs to solve captcha manually.
+                        elif result_value == ResultValue.CAPTCHA_CHALLENGE.value:
+                            raise CaptchaChallengeException("Solve CAPTCHA to continue")
+                        return  # Return empty message.
+
                     if raw:
-                        yield response
-                    if citations:
-                        yield response["item"]["messages"][1]["adaptiveCards"][0]["body"][0]["text"]
-                    yield response["item"]["messages"][1]["text"]
+                        yield response, None
+                    else:
+                        suggested_responses = None
+                        # Include list of suggested user responses, if enabled.
+                        if suggestions:
+                            suggested_responses = [
+                                item["text"]
+                                for item in messages[1]["suggestedResponses"]
+                            ]
+
+                        if citations:
+                            yield messages[1]["adaptiveCards"][0]["body"][0][
+                                "text"
+                            ], suggested_responses
+                        else:
+                            yield messages[1]["text"], suggested_responses
 
                     # Exit, type 2 is the last message.
                     streaming = False
@@ -226,20 +269,33 @@ class SydneyClient:
                     continue
                 response = json.loads(obj)
                 # Handle type 1 messages when streaming is enabled.
-                if (
-                    stream
-                    and response.get("type") == 1
-                    and response["arguments"][0].get("messages")
-                ):
+                if stream and response.get("type") == 1:
+                    messages = response["arguments"][0].get("messages")
+                    # Skip on empty response.
+                    if not messages:
+                        continue
+
                     if raw:
                         yield response
                     else:
-                        yield response["arguments"][0]["messages"][0]["text"]
+                        yield messages[0]["text"]
                 # Handle type 2 messages.
                 elif response.get("type") == 2:
+                    messages = response["item"].get("messages")
+                    if not messages:
+                        result_value = response["item"]["result"]["value"]
+                        # Throttled - raise error.
+                        if result_value == ResultValue.THROTTLED.value:
+                            raise ThrottledRequestException("Request is throttled")
+                        # Captcha chalennge - user needs to solve captcha manually.
+                        elif result_value == ResultValue.CAPTCHA_CHALLENGE.value:
+                            raise CaptchaChallengeException("Solve CAPTCHA to continue")
+                        return  # Return empty message.
+
                     if raw:
                         yield response
-                    yield response["item"]["messages"][1]["text"]
+                    else:
+                        yield messages[1]["text"]
 
                     # Exit, type 2 is the last message.
                     streaming = False
@@ -253,7 +309,15 @@ class SydneyClient:
         # Use _U cookie to create a conversation.
         cookies = {"_U": self.bing_u_cookie}
 
-        session = ClientSession(headers=HEADERS, cookies=cookies)
+        session = ClientSession(
+            headers=HEADERS,
+            cookies=cookies,
+            trust_env=self.use_proxy,  # Use `HTTP_PROXY` and `HTTPS_PROXY` environment variables.
+            connector=TCPConnector(verify_ssl=False)
+            if self.use_proxy
+            else None,  # Resolve HTTPS issue when proxy support is enabled.
+        )
+
         async with session.get(BING_CREATE_CONVESATION_URL) as response:
             if response.status != 200:
                 raise Exception(
@@ -277,8 +341,9 @@ class SydneyClient:
         self,
         prompt: str,
         citations: bool = False,
+        suggestions: bool = False,
         raw: bool = False,
-    ) -> str | dict:
+    ) -> str | dict | tuple[str | dict, list | None]:
         """
         Send a prompt to Bing Chat using the current conversation and return the answer.
 
@@ -288,6 +353,8 @@ class SydneyClient:
             The prompt that needs to be sent to Bing Chat.
         citations : bool, optional
             Whether to return any cited text. Default is False.
+        suggestions : bool, optional
+            Whether to return any suggested user responses. Default is False.
         raw : bool, optional
             Whether to return the entire response object in raw JSON format. Default is False.
 
@@ -297,8 +364,13 @@ class SydneyClient:
             The text response from Bing Chat. If citations is True, the function returns the cited text.
             If raw is True, the function returns the entire response object in raw JSON format.
         """
-        async for response in self._ask(prompt, citations, raw, stream=False):
-            return response
+        async for response, suggested_responses in self._ask(
+            prompt, citations, suggestions, raw, stream=False
+        ):
+            if suggestions:
+                return response, suggested_responses
+            else:
+                return response
 
         raise NoResponseException("No response was returned")
 
@@ -306,8 +378,9 @@ class SydneyClient:
         self,
         prompt: str,
         citations: bool = False,
+        suggestions: bool = False,
         raw: bool = False,
-    ) -> AsyncGenerator[str | dict, None]:
+    ) -> AsyncGenerator[str | dict | tuple[str | dict, list | None], None]:
         """
         Send a prompt to Bing Chat using the current conversation and stream the answer.
 
@@ -320,6 +393,8 @@ class SydneyClient:
             The prompt that needs to be sent to Bing Chat.
         citations : bool, optional
             Whether to return any cited text. Default is False.
+        suggestions : bool, optional
+            Whether to return any suggested user responses. Default is False.
         raw : bool, optional
             Whether to return the entire response object in raw JSON format. Default is False.
 
@@ -330,14 +405,19 @@ class SydneyClient:
             If raw is True, the function returns the entire response object in raw JSON format.
         """
         previous_response: str | dict = ""
-        async for response in self._ask(prompt, citations, raw, stream=True):
+        async for response, suggested_responses in self._ask(
+            prompt, citations, suggestions, raw, stream=True
+        ):
             if raw:
                 yield response
             # For text-only responses, return only newly streamed tokens.
             else:
                 new_response = response[len(previous_response) :]
                 previous_response = response
-                yield new_response
+                if suggestions:
+                    yield new_response, suggested_responses
+                else:
+                    yield new_response
 
     async def compose(
         self,
